// Copyright © 2020 Amazon Web Services
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.opengroup.osdu.notification.provider.aws.utils;

import com.amazonaws.services.cognitoidp.AWSCognitoIdentityProvider;
import com.amazonaws.services.cognitoidp.AWSCognitoIdentityProviderClientBuilder;
import com.amazonaws.services.cognitoidp.model.AdminSetUserPasswordRequest;
import com.amazonaws.services.cognitoidp.model.AdminSetUserPasswordResult;
import com.amazonaws.services.cognitoidp.model.InitiateAuthRequest;
import com.amazonaws.services.cognitoidp.model.InitiateAuthResult;
import org.opengroup.osdu.core.aws.iam.IAMConfig;

import java.util.HashMap;
import java.util.Map;

public class AwsCognitoClient {

    // Parameter value locations
    private final static String USERNAME_PARAM = "USERNAME";
    private final static String PASSWORD_PARAM = "PASSWORD";
    private final static String COGNITO_CLIENT_ID_PROPERTY = "AWS_COGNITO_CLIENT_ID";
    private final static String COGNITO_AUTH_FLOW_PROPERTY = "AWS_COGNITO_AUTH_FLOW";
    private final static String COGNITO_AUTH_PARAMS_USER_PROPERTY = "AWS_COGNITO_AUTH_PARAMS_USER";
    private final static String COGNITO_AUTH_PARAMS_PASSWORD_PROPERTY = "AWS_COGNITO_AUTH_PARAMS_PASSWORD";


    String awsCognitoClientId;
    String awsCognitoAuthFlow;
    String awsCognitoAuthParamsUser;
    String awsCognitoAuthParamsPassword;
    AWSCognitoIdentityProvider provider;


    public AwsCognitoClient(String region,String awsCognitoClientId, String awsCognitoAuthFlow, String awsCognitoAuthParamsUser
            , String awsCognitoAuthParamsPassword) {
        this.awsCognitoClientId = awsCognitoClientId;
        this.awsCognitoAuthFlow = awsCognitoAuthFlow;
        this.awsCognitoAuthParamsUser = awsCognitoAuthParamsUser;
        this.awsCognitoAuthParamsPassword = awsCognitoAuthParamsPassword;
       this.provider = generateCognitoClient(region);
    }

    public String getToken(String username, String password,String tokenType){
        Map<String, String> authParameters = new HashMap<>();
        authParameters.put(USERNAME_PARAM, username);
        authParameters.put(PASSWORD_PARAM, password);

        InitiateAuthRequest request = new InitiateAuthRequest();
        request.setClientId(awsCognitoClientId);
        request.setAuthFlow(awsCognitoAuthFlow);
        request.setAuthParameters(authParameters);
        String token="";
        InitiateAuthResult result = this.provider.initiateAuth(request);
        if(tokenType.equals("session"))
            token= result.getSession();
        else if(tokenType.equals("bearer"))
            token= "Bearer "+ result.getAuthenticationResult().getAccessToken();
        return token;
    }


    public AWSCognitoIdentityProvider getProvider() {
        return provider;
    }

    public  static AWSCognitoIdentityProvider generateCognitoClient(String region)
    {
        if (System.getenv("AWS_COGNITO_REGION") != null) {
            region = System.getenv("AWS_COGNITO_REGION");
        }
        return AWSCognitoIdentityProviderClientBuilder.standard().
                withCredentials(IAMConfig.amazonAWSCredentials()).
                withRegion(region).build();
    }

    public void setPassword(String username, String password,String userPoolId){
        AdminSetUserPasswordRequest request = new AdminSetUserPasswordRequest()
                .withUsername(username)
                .withPassword(password)
                .withPermanent(true)
                .withUserPoolId(userPoolId);
        AdminSetUserPasswordResult result = this.provider.adminSetUserPassword(request);

    }
}
