// Copyright © Microsoft Corporation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.opengroup.osdu.notification.provider.azure;

import org.opengroup.osdu.notification.provider.azure.messageBus.interfaces.ISubscriptionManager;
import org.opengroup.osdu.notification.provider.azure.messageBus.thread.ThreadScopeBeanFactoryPostProcessor;
import org.opengroup.osdu.notification.provider.azure.util.AzureServiceBusConfig;
import org.springframework.beans.factory.config.BeanFactoryPostProcessor;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.context.ApplicationContext;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.scheduling.annotation.EnableAsync;

import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

@SpringBootApplication
@ComponentScan({"org.opengroup.osdu"})
@EnableAsync
public class Application {

    public static void main(String[] args) {

        ApplicationContext context = SpringApplication.run(new Class[]{Application.class}, args);
        // Subscribe To Notification Event for Service Bus Notification Processing
        AzureServiceBusConfig azureServiceBusConfig = context.getBean(AzureServiceBusConfig.class);
        if (Boolean.parseBoolean(azureServiceBusConfig.getServiceBusEnabled())) {
            ScheduledExecutorService executorService = Executors.newSingleThreadScheduledExecutor();
            ISubscriptionManager subscriptionManager = context.getBean(ISubscriptionManager.class);
            /*
              Here the initialSubscriptionManagerDelay is used to have a delay before the first execution.
              Every consecutive execution will take place after a delay of consecutiveSubscriptionManagerDelay.
              If any of the execution exceeds the time consecutiveSubscriptionManagerDelay then next execution
              will begin immediately after the current execution is completed.
             */
            executorService.scheduleAtFixedRate(subscriptionManager, Integer.parseUnsignedInt(azureServiceBusConfig.getInitialSubscriptionManagerDelay()),
                    Integer.parseUnsignedInt(azureServiceBusConfig.getConsecutiveSubscriptionManagerDelay()), TimeUnit.SECONDS);
        }
    }

    @Bean
    public static BeanFactoryPostProcessor beanFactoryPostProcessor() {
        return new ThreadScopeBeanFactoryPostProcessor();
    }
}

