// Copyright © Schlumberger
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.opengroup.osdu.notification.provider.azure.util;

import org.opengroup.osdu.azure.logging.DependencyLogger;
import org.opengroup.osdu.azure.logging.DependencyLoggingOptions;
import org.opengroup.osdu.core.common.http.HttpResponse;
import org.opengroup.osdu.core.common.model.notification.Subscription;
import org.opengroup.osdu.notification.provider.interfaces.ISubscriberNotificationRequestLogger;
import org.springframework.context.annotation.Primary;
import org.springframework.stereotype.Component;

@Component
@Primary
public class SubscriberNotificationRequestLogger implements ISubscriberNotificationRequestLogger {

    private final String DEPENDENCY_TYPE = "Notification";
    private final String DEPENDENCY_NAME = "NotifySubscriber";
    private final DependencyLogger dependencyLogger;

    public SubscriberNotificationRequestLogger(DependencyLogger dependencyLogger) {
        this.dependencyLogger = dependencyLogger;
    }

    public void log(String notificationId, Subscription subscription, HttpResponse response) {
        final DependencyLoggingOptions loggingOptions = DependencyLoggingOptions.builder()
                .type(DEPENDENCY_TYPE)
                .name(DEPENDENCY_NAME)
                .target(subscription.getPushEndpoint())
                .timeTakenInMs(response.getLatency())
                .resultCode(response.getResponseCode())
                .success(response.isSuccessCode())
                .build();
        final String data = response.isSuccessCode()
                ? String.format("notificationId=%s subscriptionId=%s", notificationId, subscription.getId())
                : String.format("notificationId=%s subscriptionId=%s createdBy=%s response=%s", notificationId, subscription.getId(), subscription.getCreatedBy(), response.getBody());
        loggingOptions.setData(data);
        this.dependencyLogger.logDependency(loggingOptions);
    }
}
