/*
 *   Copyright 2017-2020, Schlumberger
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */

package org.opengroup.osdu.notification.provider.gcp.security;

import com.google.api.client.googleapis.auth.oauth2.GoogleIdTokenVerifier;
import com.google.api.client.http.javanet.NetHttpTransport;
import com.google.api.client.json.jackson2.JacksonFactory;
import org.springframework.stereotype.Component;

import java.util.Arrays;

@Component
public class GoogleServiceAccountValidatorGenerator {

    public GoogleIdTokenVerifier getVerifier(NetHttpTransport transport, JacksonFactory factory, String... googleAudiences) {
        GoogleIdTokenVerifier verifier;
        if (googleAudiences == null || googleAudiences.length == 0) {
            verifier = new GoogleIdTokenVerifier.Builder(transport, factory)
                    .build();
        } else {
            verifier = new GoogleIdTokenVerifier.Builder(transport, factory)
                    .setAudience(Arrays.asList(googleAudiences))
                    .build();
        }
        return verifier;
    }
}