package org.opengroup.osdu.notification.provider.gcp.service;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.RequiredArgsConstructor;
import org.opengroup.osdu.core.common.http.HttpRequest;
import org.opengroup.osdu.core.common.http.HttpResponse;
import org.opengroup.osdu.core.common.http.IHttpClient;
import org.opengroup.osdu.core.common.model.http.DpsHeaders;
import org.opengroup.osdu.core.common.model.notification.Subscription;

import java.io.IOException;
import java.util.List;

import org.opengroup.osdu.core.common.notification.SubscriptionAPIConfig;
import org.opengroup.osdu.core.common.notification.SubscriptionException;
import org.opengroup.osdu.core.common.util.UrlNormalizationUtil;
import org.springframework.stereotype.Component;

/**
 * GC extension of {@link org.opengroup.osdu.core.common.notification.SubscriptionService}.
 * Remove the current class after moving code to the original service.
 */
@Component
@RequiredArgsConstructor
public class SubscriptionServiceGc {

    private final SubscriptionAPIConfig config;
    private final IHttpClient httpClient;
    private final DpsHeadersProvider dpsHeadersProvider;

    public List<Subscription> getAll(String dataPartitionId) throws SubscriptionException {
        String url = UrlNormalizationUtil.normalizeStringUrl(config.getRootUrl(), "/subscriptions");
        DpsHeaders dpsHeaders = dpsHeadersProvider.getDpsHeaders(dataPartitionId);
        dpsHeaders.put("AppKey", config.getApiKey());
        HttpRequest request = HttpRequest.get()
                .url(url)
                .headers(dpsHeaders.getHeaders())
                .build();
        HttpResponse response = httpClient.send(request);
        return getListOfSubscriptions(response);
    }

    private List<Subscription> getListOfSubscriptions(HttpResponse response) throws SubscriptionException {
        if (response.isSuccessCode()) {
            try {
                ObjectMapper objectMapper = new ObjectMapper();
                return objectMapper.readValue(response.getBody(), new TypeReference<List<Subscription>>(){});
            } catch (IOException  ex) {
                throw new SubscriptionException("Exception in deserializing response", response, ex);
            }
        } else {
            throw new SubscriptionException("Error making request to Register service. Check the HttpResponse: " + response, response);
        }
    }
}