/*
 * Copyright 2017-2020, Schlumberger
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.opengroup.osdu.notification.provider.gcp.util;

import lombok.RequiredArgsConstructor;
import lombok.SneakyThrows;
import org.apache.http.impl.client.CloseableHttpClient;
import org.opengroup.osdu.core.gcp.googleidtoken.IGoogleIdTokenFactory;
import org.opengroup.osdu.notification.provider.interfaces.IGoogleServiceAccount;
import org.springframework.stereotype.Component;

/**
 * GSA token provider. Provides ID token for 'authorization' request header.
 * Used in HTTP notification request for GSA secret type.
 * Wraps core-lib-gcp GC ID token provider.
 */
@Component
@RequiredArgsConstructor
public class GoogleServiceAccountImpl implements IGoogleServiceAccount {

    private final IGoogleIdTokenFactory googleIdTokenFactory;
    private final CloseableHttpClient closeableHttpClient;

    @SneakyThrows
    @Override
    public String getIdToken(String keyString, String audience) {
        return this.googleIdTokenFactory.getGoogleIdToken(keyString, audience, this.closeableHttpClient);
    }
}