/*
 *  Copyright 2020-2023 Google LLC
 *  Copyright 2020-2023 EPAM Systems, Inc
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package org.opengroup.osdu.notification.provider.gcp.pubsub;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.MockitoJUnitRunner;
import org.opengroup.osdu.core.common.cache.IRedisCache;
import org.opengroup.osdu.core.common.model.notification.Subscription;
import org.opengroup.osdu.core.gcp.oqm.driver.OqmDriver;
import org.opengroup.osdu.core.gcp.oqm.model.*;
import org.opengroup.osdu.notification.provider.gcp.model.ExternalSubscriptions;
import org.opengroup.osdu.notification.provider.gcp.pubsub.receiver.OqmServiceTopicReceiver;

import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.*;

@RunWith(MockitoJUnitRunner.class)
public class OqmServiceTopicReceiverTest {

    @InjectMocks
    private OqmServiceTopicReceiver sut;
    @Mock
    private OqmAckReplier replier;
    @Mock
    private IRedisCache<String, ExternalSubscriptions> subscriptionInfoCache;
    @Mock
    private OqmDriver driver;
    @Mock
    private OqmSubscription oqmSubscription;
    @Mock
    private ExternalSubscriptions externalSubscriptions;
    private List<Subscription> subscriptionInfos;
    private OqmTopic topic;

    @Before
    public void setUp() {
        topic = OqmTopic.builder().name("topic1").build();
        Subscription subscription1 = new Subscription();
        subscription1.setTopic(topic.getName());
        Subscription subscription2 = new Subscription();
        subscription2.setTopic(topic.getName());
        Subscription subscription3 = new Subscription();
        subscription3.setTopic(topic.getName());
        subscriptionInfos = Arrays.asList(subscription1, subscription2, subscription3);
    }

    @Test
    public void testReceiveMessageSubscriptionCreated() {
        when(oqmSubscription.getTopics()).thenReturn(Collections.singletonList(topic));
        when(subscriptionInfoCache.get("tenant1")).thenReturn(externalSubscriptions);
        when(externalSubscriptions.getSubscriptions()).thenReturn(subscriptionInfos);
        OqmMessage message = OqmMessage.builder().data("Message").attributes(new HashMap<String, String>() {{
            put("data-partition-id", "tenant1");
        }}).build();
        sut.receiveMessage(message, replier);
        verify(driver, times(3)).publish(any(OqmMessage.class), any(), any());
        verify(replier, times(1)).ack();
    }
}