/*
 *   Copyright 2017-2020, Schlumberger
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */

package org.opengroup.osdu.notification.secutity;

import com.google.api.client.googleapis.auth.oauth2.GoogleIdTokenVerifier;
import com.google.api.client.http.javanet.NetHttpTransport;
import com.google.api.client.json.jackson2.JacksonFactory;
import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.opengroup.osdu.notification.provider.gcp.security.GoogleServiceAccountValidatorGenerator;
import org.powermock.modules.junit4.PowerMockRunner;

import java.util.Collection;

@RunWith(PowerMockRunner.class)
public class GoogleServiceAccountValidatorGeneratorTest {
    private final NetHttpTransport netHttpTransport = new NetHttpTransport();
    private final JacksonFactory jacksonFactory = new JacksonFactory();
    private static final String AUDIENCE_1 = "aud1";
    private static final String AUDIENCE_2 = "aud2";

    @InjectMocks
    private GoogleServiceAccountValidatorGenerator sut;

    @Test
    public void should_returnVerifierWithoutAudiences_when_noAudiencesProvided() {
        GoogleIdTokenVerifier verifier = this.sut.getVerifier(netHttpTransport, jacksonFactory);
        Assert.assertNull(verifier.getAudience());
    }

    @Test
    public void should_returnVerifierWithAudiences_when_AudiencesProvided() {
        GoogleIdTokenVerifier verifier = this.sut.getVerifier(netHttpTransport, jacksonFactory, AUDIENCE_1, AUDIENCE_2);
        Collection<String> audiences = verifier.getAudience();
        Assert.assertTrue(audiences.contains(AUDIENCE_1));
        Assert.assertTrue(audiences.contains(AUDIENCE_2));
    }
}
