/*
 *  Copyright 2020-2023 Google LLC
 *  Copyright 2020-2023 EPAM Systems, Inc
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package org.opengroup.osdu.notification.util;

import com.google.gson.Gson;
import com.sun.jersey.api.client.Client;
import com.sun.jersey.api.client.ClientResponse;
import com.sun.jersey.api.client.WebResource;
import org.apache.commons.lang3.StringUtils;

import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;
import javax.ws.rs.core.MediaType;
import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.net.HttpURLConnection;
import java.net.URI;
import java.security.SecureRandom;
import java.security.cert.X509Certificate;
import java.util.Arrays;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;

import static org.junit.Assert.assertEquals;

public class HttpClient {
  private HttpClient(){
  }

  public static ClientResponse send(String url, String path, String httpMethod,
                                    Map<String, String> headers,
                                    String requestBody, String query) throws Exception {

    String normalizedUrl = new URI(String.format("%s/%s", url, path)).normalize().toString();
    normalizedUrl = StringUtils.removeEnd(normalizedUrl, "/");
    log(httpMethod, normalizedUrl + query, headers, requestBody);
    Client client = getClient();

    WebResource webResource = client.resource(normalizedUrl + query);
    WebResource.Builder builder = webResource.accept(MediaType.APPLICATION_JSON)
        .type(MediaType.APPLICATION_JSON);
    headers.forEach(builder::header);

    if ("POST".equals(httpMethod) && StringUtils.isEmpty(requestBody)) {
      requestBody = "{}"; //solves 411 error when sending empty-body POST request
    }

    return builder.method(httpMethod, ClientResponse.class, requestBody);
  }

  private static void log(String method, String url, Map<String, String> headers, String body) {
    System.out.println(String.format("%s: %s", method, url));
    System.out.println(body);
  }

  @SuppressWarnings("unchecked")
  public static <T> T getResult(ClientResponse response, int exepectedStatus, Class<T> classOfT) {
    assertEquals(exepectedStatus, response.getStatus());
    if (exepectedStatus == 204) {
      return null;
    }

    assertEquals("application/json; charset=UTF-8", response.getType().toString());
    String json = response.getEntity(String.class);
    if (classOfT == String.class) {
      return (T) json;
    }

    Gson gson = new Gson();
    return gson.fromJson(json, classOfT);
  }

  protected static Client getClient() {
    TrustManager[] trustAllCerts = new TrustManager[]{new X509TrustManager() {
      @Override
      public X509Certificate[] getAcceptedIssuers() {
        return null;
      }

      @Override
      public void checkClientTrusted(X509Certificate[] certs, String authType) {
      }

      @Override
      public void checkServerTrusted(X509Certificate[] certs, String authType) {
      }
    }};

    try {
      SSLContext sc = SSLContext.getInstance("TLS");
      sc.init(null, trustAllCerts, new SecureRandom());
      HttpsURLConnection.setDefaultSSLSocketFactory(sc.getSocketFactory());
    } catch (Exception e) {
    }
    allowMethods("PATCH");
    return Client.create();
  }

  private static void allowMethods(String... methods) {
    try {
      Field methodsField = HttpURLConnection.class.getDeclaredField("methods");

      Field modifiersField = Field.class.getDeclaredField("modifiers");
      modifiersField.setAccessible(true);
      modifiersField.setInt(methodsField, methodsField.getModifiers() & ~Modifier.FINAL);

      methodsField.setAccessible(true);

      String[] oldMethods = (String[]) methodsField.get(null);
      Set<String> methodsSet = new LinkedHashSet<>(Arrays.asList(oldMethods));
      methodsSet.addAll(Arrays.asList(methods));
      String[] newMethods = methodsSet.toArray(new String[0]);

      methodsField.set(null/*static field*/, newMethods);
    } catch (NoSuchFieldException | IllegalAccessException e) {
      throw new IllegalStateException(e);
    }
  }
}
