/*
 *  Copyright 2020-2023 Google LLC
 *  Copyright 2020-2023 EPAM Systems, Inc
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package org.opengroup.osdu.notification.util;

import com.sun.jersey.api.client.ClientResponse;
import org.opengroup.osdu.core.common.model.http.DpsHeaders;

import java.util.HashMap;
import java.util.Map;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

public class ServicesUtils {
  private String storageHost;
  private String legalHost;
  private TestUtils testUtils;
  private String partitionId;
  private FileUtils fileUtils;

  public ServicesUtils(String storageHost, String legalHost, TestUtils testUtils, String partitionId) {
    this.storageHost = storageHost;
    this.legalHost = legalHost;
    this.testUtils = testUtils;
    this.partitionId = partitionId;
    this.fileUtils = new FileUtils();
  }

  public ClientResponse createLegalTag(String tagName) throws Exception {
    String legalBody = fileUtils.readFromLocalFilePath("/LegalTag.json");
    legalBody = legalBody.replace("{{tagName}}", tagName);
    Map<String, String> headers = new HashMap<>();
    headers.put(DpsHeaders.DATA_PARTITION_ID, partitionId);
    headers.put(DpsHeaders.AUTHORIZATION, testUtils.getAdminToken());

    ClientResponse legalResponse = HttpClient.send(legalHost, "legaltags", "POST", headers, legalBody, "");

    boolean createdOrAlreadyExists = legalResponse.getStatus() == 201 || legalResponse.getStatus() == 409;
    assertTrue(createdOrAlreadyExists);
    return legalResponse;
  }

  public void deleteLegalTag(String tagName) throws Exception{
    Map<String, String> headers = new HashMap<>();
    headers.put(DpsHeaders.DATA_PARTITION_ID, partitionId);
    headers.put(DpsHeaders.AUTHORIZATION, testUtils.getAdminToken());
    ClientResponse legalResponse = HttpClient.send(legalHost, String.format("legaltags/%s", tagName), "DELETE",
        headers, "", "");
    assertEquals(204, legalResponse.getStatus());
  }

  public void createStorageRecords(String suffix, int count, String legalTag) throws Exception{
    String body = fileUtils.readFromLocalFilePath("/StorageRecord.json");
    body = body.replace("{{data-partition-id}}", partitionId);
    body = body.replace("{{legal-tag}}", partitionId + "-" + legalTag);
    for (int i = 0; i < count; i++) {
      String actualBody = body.replace("{{ids-suffix}}", suffix + String.valueOf(i));
      Map<String, String> headers = new HashMap<>();
      headers.put(DpsHeaders.DATA_PARTITION_ID, partitionId);
      headers.put(DpsHeaders.AUTHORIZATION, testUtils.getAdminToken());
      ClientResponse storageResponse = HttpClient.send(storageHost, "records", "PUT",
          headers, actualBody, "");
      assertEquals(201, storageResponse.getStatus());
    }
  }

  public void deleteStorageRecords(int count, String suffix) throws Exception{
    for (int i = 0; i < count; i++) {
      Map<String, String> headers = new HashMap<>();
      headers.put(DpsHeaders.DATA_PARTITION_ID, partitionId);
      headers.put(DpsHeaders.AUTHORIZATION, testUtils.getAdminToken());
      String recordId = partitionId + ":dataset--ConnectedSource.Generic:notification-test-" + suffix + i;
      ClientResponse storageResponse = HttpClient.send(storageHost, "records/" + recordId, "DELETE",
          headers, "", "");
      assertEquals(204, storageResponse.getStatus());
    }
  }

}
